package com.gic.haoban.manage.service.service.out.impl.hm;

import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.PageResult2;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeListDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeStoreDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmClerkRelationQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeListQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeQDTO;
import com.gic.haoban.manage.api.service.hm.HmQrcodeApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.pojo.bo.hm.HmClerkRelationBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.hm.HmClerkRelationService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.gic.haoban.manage.service.util.QwFriendLimitCountUtil;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactDTO;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactResultDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author mozhu
 * @date 2022/7/5 09:34
 */
@Service
public class HmQrcodeApiServiceImpl implements HmQrcodeApiService {

    private static Logger logger = LoggerFactory.getLogger(HmQrcodeApiServiceImpl.class);

    @Autowired
    private HmQrcodeService hmQrcodeService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private Config config;
    @Autowired
    private StaffClerkRelationService staffClerkRelationService;
    @Autowired
    private HmClerkRelationService hmClerkRelationService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private StaffService staffService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int add(HmQrcodeQDTO hmQrcodeQDTO) {
        String wxEnterpriseId = hmQrcodeQDTO.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            logger.error("未查询到微信企业");
            return 0;
        }

        hmQrcodeQDTO.setHmId(UniqueIdUtils.uniqueLong());
        Integer hmType = hmQrcodeQDTO.getHmType();
        Long hmId = hmQrcodeQDTO.getHmId();
        if (hmType == 1) {
            hmQrcodeQDTO.setHmCode("HM01" + hmId);
        } else {
            hmQrcodeQDTO.setHmCode("HM02" + hmId);
        }

        List<String> clerkIdList = setStoreClerk(hmQrcodeQDTO, wxEnterpriseId, hmId, hmType);

        //生成活码
        QywxExternalcontactDTO qywxExternalcontactDTO = new QywxExternalcontactDTO();
        qywxExternalcontactDTO.setType(hmQrcodeQDTO.getHmType());
        qywxExternalcontactDTO.setScene(2);
        qywxExternalcontactDTO.setRemark(hmQrcodeQDTO.getName());
        qywxExternalcontactDTO.setSkipVerify(hmQrcodeQDTO.getPassFlag() == 1);
        qywxExternalcontactDTO.setState("HM" + hmId);
        List<String> userIds = staffClerkRelationService.listWxUserIdByClerkIds(clerkIdList, wxEnterpriseId);
        qywxExternalcontactDTO.setUser(userIds);
        QywxExternalcontactResultDTO qywxExternalcontactResultDTO = qywxUserApiService.addContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxExternalcontactDTO);
        if (qywxExternalcontactResultDTO != null && "0".equals(qywxExternalcontactResultDTO.getErrcode())) {
            hmQrcodeQDTO.setWxQrcode(qywxExternalcontactResultDTO.getQr_code());
            hmQrcodeQDTO.setWxConfigId(qywxExternalcontactResultDTO.getConfig_id());
        }
        return hmQrcodeService.insert(hmQrcodeQDTO);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public int update(HmQrcodeQDTO hmQrcodeQDTO) {
        String wxEnterpriseId = hmQrcodeQDTO.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            logger.error("未查询到微信企业");
            return 0;
        }
        Long hmId = hmQrcodeQDTO.getHmId();
        Integer hmType = hmQrcodeQDTO.getHmType();

        List<String> clerkIdList = setStoreClerk(hmQrcodeQDTO, wxEnterpriseId, hmId, hmType);

        //更新活码
        QywxExternalcontactDTO qywxExternalcontactDTO = new QywxExternalcontactDTO();
        qywxExternalcontactDTO.setType(hmQrcodeQDTO.getHmType());
        qywxExternalcontactDTO.setScene(2);
        qywxExternalcontactDTO.setRemark(hmQrcodeQDTO.getName());
        qywxExternalcontactDTO.setSkipVerify(hmQrcodeQDTO.getPassFlag() == 1);
        qywxExternalcontactDTO.setState("HM" + hmId);
        List<String> userIds = staffClerkRelationService.listWxUserIdByClerkIds(clerkIdList, wxEnterpriseId);
        qywxExternalcontactDTO.setUser(userIds);
        boolean flag = qywxUserApiService.updateContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxExternalcontactDTO);
        if (!flag) {
            logger.error("更新企微活码失败");
            return 0;
        }
        return hmQrcodeService.update(hmQrcodeQDTO);
    }

    /**
     * 设置门店和导购
     *
     * @param hmQrcodeQDTO
     * @param wxEnterpriseId
     * @param hmId
     * @param hmType
     * @return
     */
    private List<String> setStoreClerk(HmQrcodeQDTO hmQrcodeQDTO, String wxEnterpriseId, Long hmId, Integer hmType) {
        List<String> clerkIdList = hmQrcodeQDTO.getClerkIdList();
        String storeId = hmQrcodeQDTO.getStoreId();
        StoreDTO storeDTO = storeService.getStore(storeId);
        if (storeDTO != null) {
            storeDTO.setStoreCode(storeDTO.getStoreCode());
            storeDTO.setStoreName(storeDTO.getStoreName());
        }

        if (hmType == 2) {
            //多人活码
            insertHmClerkRelation(hmQrcodeQDTO, wxEnterpriseId, hmId, clerkIdList);
        } else if (hmType == 1) {
            //单人活码
            String clerkId = clerkIdList.get(0);
            HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryByClerkId(clerkId, wxEnterpriseId);
            if (hmQrcodeBO != null) {
                logger.error("该导购已经生成过活码：{}", clerkId);
                return null;
            }
            hmQrcodeQDTO.setClerkId(clerkId);
            ClerkDTO clerkDTO = clerkService.getclerkById(clerkId);
            if (clerkDTO != null) {
                hmQrcodeQDTO.setClerkName(clerkDTO.getClerkName());
                hmQrcodeQDTO.setClerkCode(clerkDTO.getClerkCode());
            }
        }
        return clerkIdList;
    }

    /**
     * 多人活码关联关系
     *
     * @param hmQrcodeQDTO
     * @param wxEnterpriseId
     * @param hmId
     * @param clerkIdList
     */
    private void insertHmClerkRelation(HmQrcodeQDTO hmQrcodeQDTO, String wxEnterpriseId, Long hmId, List<String> clerkIdList) {
        List<HmClerkRelationQDTO> clerkRelationQDTOS = new ArrayList<>();
        for (String clerkId : clerkIdList) {
            StaffClerkRelationDTO clerkRelationDTO = staffClerkRelationService.getByClerkId(clerkId);
            HmClerkRelationQDTO hmClerkRelationQDTO = new HmClerkRelationQDTO();
            hmClerkRelationQDTO.setRelationId(UniqueIdUtils.uniqueLong());
            hmClerkRelationQDTO.setHmId(hmId);
            hmClerkRelationQDTO.setWxUserId(clerkRelationDTO.getWxUserId());
            hmClerkRelationQDTO.setWxEnterpriseId(wxEnterpriseId);
            hmClerkRelationQDTO.setEnterpriseId(hmQrcodeQDTO.getEnterpriseId());
            hmClerkRelationQDTO.setStoreId(hmQrcodeQDTO.getStoreId());
            hmClerkRelationQDTO.setStoreName(hmQrcodeQDTO.getStoreName());
            hmClerkRelationQDTO.setStoreCode(hmQrcodeQDTO.getStoreCode());
            hmClerkRelationQDTO.setClerkId(clerkId);
            ClerkDTO clerkDTO = clerkService.getclerkById(clerkId);
            if (clerkDTO != null) {
                hmClerkRelationQDTO.setClerkName(clerkDTO.getClerkName());
                hmClerkRelationQDTO.setClerkCode(clerkDTO.getClerkCode());
            }
            clerkRelationQDTOS.add(hmClerkRelationQDTO);
        }
        hmClerkRelationService.insertBatch(clerkRelationQDTOS);
    }

    @Override
    public PageResult2<HmQrcodeListDTO> queryListByPage(HmQrcodeListQDTO hmQrcodeListQDTO) {
        return PageUtil.getPageInfo(hmQrcodeService.queryByPage(hmQrcodeListQDTO));
    }

    @Override
    public HmQrcodeDTO queryById(Long hmId) {
        return EntityUtil.changeEntityNew(HmQrcodeDTO.class, hmQrcodeService.queryById(hmId));
    }

    @Override
    public boolean delById(Long hmId) {
        HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryById(hmId);
        if (hmQrcodeBO == null) {
            return false;
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(hmQrcodeBO.getWxEnterpriseId());
        boolean flag = qywxUserApiService.delContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), hmQrcodeBO.getWxConfigId());
        if (flag) {
            hmQrcodeService.deleteById(hmId);
        }
        return true;
    }

    @Override
    public boolean delByClerkId(String clerkId, String enterpriseId) {
        // 单人活码 直接废弃，多人活码，更新
        HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryByClerkId(clerkId, enterpriseId);
        if (hmQrcodeBO != null) {
            delById(hmQrcodeBO.getHmId());
        }

        // 多人活码逻辑
        List<HmClerkRelationBO> hmClerkRelationBOS = hmClerkRelationService.queryByClerkId(clerkId, enterpriseId);
        if (CollectionUtils.isEmpty(hmClerkRelationBOS)) {
            logger.info("无多人活码");
            return true;
        }
        for (HmClerkRelationBO hmClerkRelationBO : hmClerkRelationBOS) {
            Long hmId = hmClerkRelationBO.getHmId();
            List<String> clerkIds = hmClerkRelationService.getClerkIdByHmId(hmId);
            clerkIds.remove(clerkId);
            // 更新活码
            HmQrcodeQDTO hmQrcodeQDTO = new HmQrcodeQDTO();
            hmQrcodeQDTO.setHmId(hmId);
            hmQrcodeQDTO.setClerkIdList(clerkIds);
            update(hmQrcodeQDTO);
        }
        return true;
    }

    @Override
    public int updateAddNumById(Long hmId, String clerkId) {
        HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryById(hmId);
        if (hmQrcodeBO == null) {
            logger.error("无活码数据");
            return 0;
        }
        Integer hmType = hmQrcodeBO.getHmType();
        int limit = QwFriendLimitCountUtil.getLimit(true);
        Integer addNum = hmQrcodeBO.getAddNum();
        if (hmType == 1) {
            //单人活码
            Integer overFlag = 0;
            if (addNum + 1 >= limit) {
                overFlag = 1;
            }
            return hmQrcodeService.updateAddNumById(hmId, overFlag);
        } else if (hmType == 2) {
            //多人活码
            HmClerkRelationBO hmClerkRelationBO = hmClerkRelationService.queryByHmIdAndClerkId(hmId, clerkId);
            if (hmClerkRelationBO == null) {
                logger.error("无多人活码导购数据,clerkId:{},hmId:{}", clerkId, hmId);
                return 0;
            }
            Integer overFlag = 0;
            if (addNum + 1 >= limit) {
                overFlag = 1;
            }
            Long relationId = hmClerkRelationBO.getRelationId();
            return hmClerkRelationService.updateAddNumById(relationId, overFlag);
        }
        return 0;
    }

    @Override
    public void resetAddNumAndOverFlag(String param) {
        logger.info("重置上限时间定时任务 start");
        hmQrcodeService.resetAddNumAndOverFlag();
        hmClerkRelationService.resetAddNumAndOverFlag();
        logger.info("重置上限时间定时任务 end");
    }

    @Override
    public void checkStaffOpenContact(String param) {
        logger.info("检查员工是否开启联系我,定时任务，start");
        List<TabHaobanWxEnterprise> tabHaobanWxEnterprises = wxEnterpriseService.listAll();
        for (TabHaobanWxEnterprise tabHaobanWxEnterpris : tabHaobanWxEnterprises) {
            //查询出所有正常的导购
            String wxEnterpriseId = tabHaobanWxEnterpris.getWxEnterpriseId();
            //企微已经开启联系我
            String wxUserIds = qywxUserApiService.listCorpExternalUser(tabHaobanWxEnterpris.getCorpid(), config.getWxSuiteid());
            String[] split = wxUserIds.split(",");
            List<String> wxUserIdsList = Arrays.asList(split);
            List<StaffClerkRelationDTO> staffClerkRelationDTOS = staffClerkRelationService.listIdsByWxOpenUserIds(wxUserIdsList, wxEnterpriseId);
            if (CollectionUtils.isEmpty(staffClerkRelationDTOS)) {
                logger.error("无导购关联数据");
                return;
            }
            for (StaffClerkRelationDTO staffClerkRelationDTO : staffClerkRelationDTOS) {
                //具有联系我功能的导购创建活码
                staffClerkRelationService.updateOpenConcatFlagById(1, staffClerkRelationDTO.getStaffClerkRelationId());
                //创建活码
                HmQrcodeQDTO hmQrcodeQDTO = new HmQrcodeQDTO();
                hmQrcodeQDTO.setHmType(1);
                hmQrcodeQDTO.setWxEnterpriseId(wxEnterpriseId);
                hmQrcodeQDTO.setEnterpriseId(staffClerkRelationDTO.getEnterpriseId());
                hmQrcodeQDTO.setName(staffClerkRelationDTO.getStaffName());
                hmQrcodeQDTO.setPassFlag(1);
                hmQrcodeQDTO.setStoreId(staffClerkRelationDTO.getStoreId());
                hmQrcodeQDTO.setClerkIdList(Collections.singletonList(staffClerkRelationDTO.getClerkId()));
                add(hmQrcodeQDTO);
            }

            List<StaffClerkRelationDTO> relationDTOS = staffClerkRelationService.listIdsByNotInWxOpenUserIds(wxUserIdsList, wxEnterpriseId);
            if (CollectionUtils.isEmpty(relationDTOS)) {
                logger.error("无导购不包含关联数据");
                return;
            }
            for (StaffClerkRelationDTO relationDTO : relationDTOS) {
                //有活码，但是企微联系我功能 被关闭
                String clerkId = relationDTO.getClerkId();
                HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryByClerkId(clerkId, relationDTO.getWxEnterpriseId());
                if (hmQrcodeBO == null) {
                    logger.info("导购无活码，无需处理：clerkId：{}", clerkId);
                    continue;
                }
                hmQrcodeService.updateStatusById(hmQrcodeBO.getHmId(), 3);
            }
        }
        logger.info("检查员工是否开启联系我,定时任务，end");
    }

    @Override
    public List<HmQrcodeStoreDTO> getHmStoreByEnterpriseId(String enterpriseId) {
        String key = "HM:STORE:" + enterpriseId;
        Object cache = RedisUtil.getCache(key);
        if (cache != null) {
            return (List<HmQrcodeStoreDTO>) cache;
        }
        List<HmQrcodeStoreDTO> hmQrcodeStoreDTOS = EntityUtil.changeEntityListNew(HmQrcodeStoreDTO.class, hmQrcodeService.getHmStoreByEnterpriseId(enterpriseId));
        RedisUtil.setCache(key, hmQrcodeStoreDTOS, 10L, TimeUnit.MINUTES);
        return hmQrcodeStoreDTOS;
    }
}
