package com.gic.haoban.manage.web.controller.hm;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.base.api.common.PageResult2;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.qdto.hm.HmPageQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmPageRelationQDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageRelationDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageDTO;
import com.gic.haoban.manage.api.service.hm.HmPageApiService;
import com.gic.haoban.manage.web.vo.hm.HmPageInfoVO;
import com.gic.haoban.manage.web.vo.hm.HmPageListVO;
import com.gic.haoban.manage.web.vo.hm.HmPageSettingReq;
import com.gic.haoban.manage.web.vo.hm.PageRelationVO;
import com.gic.log.record.anno.GicLogRecord;
import com.gic.log.record.util.GicLogRecordEvaluationContext;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author MUSI
 * @Date 2022/7/4 1:58 PM
 * @Description
 * @Version
 **/
@RestController
@RequestMapping(path = "/hm/guide")
public class HmPageController {

    @Autowired
    private HmPageApiService pageApiService;

    @RequestMapping(path = "/page/save")
    public RestResponse<?> savePage(@Valid HmPageSettingReq pageSettingVO){
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        HmPageDTO pageDTO = new HmPageDTO();
        pageDTO.setPageName(pageSettingVO.getTemplateName());
        pageDTO.setPageTitle(pageSettingVO.getPageTitle());
        pageDTO.setMerchantName(pageSettingVO.getMerchantName());
        pageDTO.setMerchantPhoneNumber(pageSettingVO.getMerchantPhoneNumber());
        pageDTO.setMerchantLogo(pageSettingVO.getMerchantLogo());
        pageDTO.setPageBackgroundImg(pageSettingVO.getPageBackgroundImg());
        pageDTO.setGuideComment(pageSettingVO.getGuideComment());
        pageDTO.setCreateorId(loginUser.getClerkId());
        pageDTO.setCreateorName(loginUser.getClerkName());
        pageDTO.setEnterpriseId(loginUser.getEnterpriseId());
        pageDTO.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        ServiceResponse<Long> serviceResponse = pageApiService.saveOrUpdateHmPage(pageDTO);
        if (serviceResponse.isSuccess()){
            String logContent = "新增落地页模板【MB01" + serviceResponse.getResult()+"-" + pageSettingVO.getTemplateName() +"】";
            return RestResponse.successResult(serviceResponse.getResult());
        }
        GicLogRecordEvaluationContext.putAttribute("saved", Boolean.FALSE);
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }


    @RequestMapping(path = "/page/edit")
    public RestResponse<?> editPageTemplate(@Valid HmPageSettingReq pageSettingVO){
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        HmPageDTO pageDTO = new HmPageDTO();
        pageDTO.setPageId(pageSettingVO.getTemplateId());
        pageDTO.setPageName(pageSettingVO.getTemplateName());
        pageDTO.setPageTitle(pageSettingVO.getPageTitle());
        pageDTO.setMerchantName(pageSettingVO.getMerchantName());
        pageDTO.setMerchantPhoneNumber(pageSettingVO.getMerchantPhoneNumber());
        pageDTO.setMerchantLogo(pageSettingVO.getMerchantLogo());
        pageDTO.setPageBackgroundImg(pageSettingVO.getPageBackgroundImg());
        pageDTO.setGuideComment(pageSettingVO.getGuideComment());
        pageDTO.setCreateorId(loginUser.getClerkId());
        pageDTO.setCreateorName(loginUser.getClerkName());
        pageDTO.setEnterpriseId(loginUser.getEnterpriseId());
        pageDTO.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        ServiceResponse<HmPageDTO> queryPageDetail = pageApiService.queryPageDetail(pageDTO.getPageId());
        ServiceResponse<Long> serviceResponse = pageApiService.saveOrUpdateHmPage(pageDTO);
        if (serviceResponse.isSuccess()){
            String logContent = this.buildEditPageContent(pageDTO, queryPageDetail.getResult());
            return RestResponse.successResult(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    /**
     * 编辑落地页模板【模板ID-模板名称】：模板名称「 模板旧名称」变更「 模板新名称」，
     * 页面标题「 页面旧标题」便跟「 页面新标题」，
     * 商户名称「 商户名称」变更「 商户名称」，
     * 商户电话「 商户电话」变更「 商户电话」，
     * 商户LOGO、页面背景、
     * 引导语「 引导语」变更「 引导语」
     * @param newPageDTO
     * @param oldPageDTO
     * @return
     */
    private String buildEditPageContent(HmPageDTO newPageDTO, HmPageDTO oldPageDTO){
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("编辑落地页模板【")
                .append(oldPageDTO.getPageCode())
                .append("-")
                .append(oldPageDTO.getPageName())
                .append("：模板名称「 ")
                .append(oldPageDTO.getPageName())
                .append("」变更为「 ")
                .append(newPageDTO.getPageName())
                .append("」，页面标题「 ")
                .append(oldPageDTO.getPageTitle())
                .append("」变更为「")
                .append(newPageDTO.getPageTitle())
                .append("」，商户名称「")
                .append(oldPageDTO.getMerchantName())
                .append("」变更为「 ")
                .append(newPageDTO.getMerchantName())
                .append("」，商户电话「 ")
                .append(oldPageDTO.getMerchantPhoneNumber())
                .append("」变更为「")
                .append(newPageDTO.getMerchantPhoneNumber())
                .append("」，引导语「 ")
                .append(oldPageDTO.getGuideComment())
                .append("」变更为「")
                .append(newPageDTO.getGuideComment())
                .append("」");
        return stringBuilder.toString();
    }

    @RequestMapping(path = "/page/list")
    public RestResponse<?> queryPageList(HmPageListVO pageListVO){

        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        HmPageQDTO hmPageQDTO = new HmPageQDTO();
        hmPageQDTO.setPageNum(pageListVO.getPageNum());
        hmPageQDTO.setPageSize(pageListVO.getPageSize());
        hmPageQDTO.setEnterpriseId(loginUser.getEnterpriseId());
        hmPageQDTO.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        hmPageQDTO.setPageSearchText(pageListVO.getPageSearchText());
        hmPageQDTO.setStartTime(pageListVO.getStartTime());
        hmPageQDTO.setEndTime(pageListVO.getEndTime());
        ServiceResponse<Page<HmPageDTO>> serviceResponse = pageApiService.queryPageList(hmPageQDTO);
        PageResult2 pageInfo = PageUtil.getPageInfo(serviceResponse.getResult());
        List<HmPageInfoVO> pageInfoVos = ((List<HmPageDTO>)pageInfo.getList()).stream().map(item -> {
            HmPageInfoVO pageInfoVo = new HmPageInfoVO();
            BeanUtils.copyProperties(item, pageInfoVo);
            pageInfoVo.setTemplateName(item.getPageName());
            pageInfoVo.setTemplateCode(item.getPageCode());
            pageInfoVo.setTemplateId(item.getPageId());
            return pageInfoVo;
        }).collect(Collectors.toList());
        pageInfo.setList(pageInfoVos);
        return RestResponse.successResult(pageInfo);
    }

    @RequestMapping(path = "/page/delete")
    public RestResponse<?> deletePage(Long templateId){
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        HmPageDTO hmPageDTO = new HmPageDTO();
        hmPageDTO.setPageId(templateId);
        hmPageDTO.setModifierId(loginUser.getClerkId());
        hmPageDTO.setModifierName(loginUser.getClerkName());

        ServiceResponse<HmPageDTO> queryPageDetail = pageApiService.queryPageDetail(templateId);
        ServiceResponse<Boolean> serviceResponse = pageApiService.deleteHmPage(hmPageDTO);
        if (serviceResponse.isSuccess()){
            String logContent = "删除落地页模板【" + queryPageDetail.getResult().getPageTitle() + "-" + queryPageDetail.getResult().getPageName()+"】";

            return RestResponse.successResult(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    @RequestMapping(path = "/page/detail")
    public RestResponse<?> queryPageDetail(Long templateId){
        ServiceResponse<HmPageDTO> serviceResponse = pageApiService.queryPageDetail(templateId);
        return RestResponse.successResult(serviceResponse.getResult());
    }

    @RequestMapping(path = "/page/relation")
    public RestResponse<?> queryPageRelationList(PageRelationVO pageRelationVO){
        HmPageRelationQDTO pageRelationQueryDTO = new HmPageRelationQDTO();
        pageRelationQueryDTO.setPageNum(pageRelationVO.getPageNum());
        pageRelationQueryDTO.setPageSize(pageRelationVO.getPageSize());
        pageRelationQueryDTO.setSearchText(pageRelationVO.getSearchContent());
        pageRelationQueryDTO.setPageId(pageRelationQueryDTO.getPageId());
        ServiceResponse<Page<HmPageRelationDTO>> serviceResponse = pageApiService.queryPageRelation(pageRelationQueryDTO);
        PageResult2 pageInfo = PageUtil.getPageInfo(serviceResponse.getResult());
        return RestResponse.successResult(pageInfo);
    }
}
